# -*- coding: utf-8 -*-
"""
This class is aim to install lamp server(Linux, Apache, Mysql and PHP) into your
local environment.

Usage:
    $ sudo pip install ezhost
    $ ezhost -s lamp -H 127.0.0.1:2201 -U vagrant -P vagrant

Author: Zhe Xiao
Github: https://github.com/zhexiao/ezhost.git
"""

from io import StringIO

from ezhost.ServerCommon import ServerCommon

# fabric libs
from fabric.colors import red, green
from fabric.api import prompt, sudo, put
from fabric.state import output

# hide exec command
output['running'] = False


class ServerLamp(ServerCommon):

    def __init__(self, args, **kwargs):
        self.args = args

    def install(self):
        self.update_sys()
        self.install_apache()
        self.install_mysql()
        self.install_php()

    def update_sys(self):
        if self.args.force or prompt(red(' * Update system package (y/n)?'), default='y') == 'y':
            self.common_update_sys()

    def install_apache(self):
        if self.args.force or prompt(red(' * Install Apache2 (y/n)?'), default='y') == 'y':
            self.common_install_apache2()

    def install_mysql(self):
        if self.args.force or prompt(red(' * Install MySql (y/n)?'), default='y') == 'y':
            self.common_install_mysql()

    def install_php(self):
        if self.args.force or prompt(red(' * Install PHP (y/n)?'), default='y') == 'y':
            try:
                sudo('apt-get install php5 php5-cli php5-mysql php5-gd php5-curl libapache2-mod-php5 php5-mcrypt -y')

                # do apache config
                put(StringIO(self.apache_dir_index), '/etc/apache2/mods-enabled/dir.conf', use_sudo=True)

                sudo('service apache2 restart')
                print(green(' * Installed php5 and php5-mysql in the system.'))
            except:
                print(red(' * Install php5 and php5-mysql failed.'))

            # write phpinfo for test
            put(StringIO(self.phpinfo), '{0}/info.php'.format(self.apache_web_dir), use_sudo=True)

            print(green(' * Done'))
            print()



supervisor_dir = '/etc/supervisor'
supervisor_conf_dir = '{0}/conf.d'.format(supervisor_dir)

uwsgi_ini_conf = """[uwsgi]
chdir = {0}/{1}
home = {2}
module = {1}.wsgi:application

uid = {3}
gid = www-data

master = true
processes = 5

socket = /tmp/{1}.sock
chmod-socket = 664
vacuum = true
"""

uwsgi_nginx_conf = """server {{
    listen 80;
    server_name localhost;

    location = /favicon.ico {{
        access_log off;
        log_not_found off;
    }}

    location / {{
        include uwsgi_params;
        uwsgi_pass unix:/tmp/{0}.sock;
    }}
}}
"""

uwsgi_supervisor_conf = """[program:{0}]
command=uwsgi --ini {1}/{0}.ini
directory={1}
numprocs=1
stdout_logfile=/var/log/{0}_out.log
stderr_logfile=/var/log/{0}_error.log
autostart=true
autorestart=true
startsecs=2
stopwaitsecs=2
killasgroup=true
priority=998
"""



# -*- coding: utf-8 -*-
"""
This class is used for create the common command such as mysql install, nginx install
and so on..

Author: Zhe Xiao
Github: https://github.com/zhexiao/ezhost.git
"""

from io import StringIO

from ezhost.ServerAbstract import ServerAbstract
import ezhost.config.bigdata_conf_string as bigdata_conf

# fabric libs
from fabric.colors import red, green
from fabric.api import prompt, run, sudo, put, env
from fabric.contrib.files import exists, append, comment, sed, uncomment
from fabric.context_managers import cd


class ServerCommon(ServerAbstract):
    """
        All common function will create inside this class.
    """

    def prompt_check(self, message):
        message += ' (y/n)'
        p_signal = prompt(red(message), default='n')
        if p_signal == 'y':
            return True
        return False

    def common_update_sys(self):
        """
            update system package
        """
        try:
            sudo('apt-get update -y')
        except Exception as e:
            print(e)

        print(green('System package is up to date.'))
        print()

    def common_install_mysql(self):
        """
            Install mysql
        """
        sudo("debconf-set-selections <<< 'mysql-server mysql-server/root_password password {0}'".format(self.mysql_password))
        sudo("debconf-set-selections <<< 'mysql-server mysql-server/root_password_again password {0}'".format(self.mysql_password))
        sudo('apt-get install mysql-server -y')

        print(green(' * Installed MySql server in the system.'))
        print(green(' * Done'))
        print()

    def common_install_nginx(self):
        """
            Install nginx
        """
        run('echo "deb http://ppa.launchpad.net/nginx/stable/ubuntu $(lsb_release -sc) main" | sudo tee /etc/apt/sources.list.d/nginx-stable.list')
        sudo('apt-key adv --keyserver keyserver.ubuntu.com --recv-keys C300EE8C')
        sudo('apt-get update -y')
        sudo('apt-get install nginx -y')

        print(green(' * Installed Nginx in the system.'))
        print(green(' * Done'))
        print()

    def common_config_nginx_ssl(self):
        """
            Convert nginx server from http to https
        """
        if prompt(red(' * Change url from http to https (y/n)?'), default='n') == 'y':
            if not exists(self.nginx_ssl_dir):
                sudo('mkdir -p {0}'.format(self.nginx_ssl_dir))

            # generate ssh key
            sudo('openssl req -x509 -nodes -days 365 -newkey rsa:2048 -keyout {0}/cert.key -out {0}/cert.pem'.format(self.nginx_ssl_dir))

            # do nginx config config
            put(StringIO(self.nginx_web_ssl_config), '/etc/nginx/sites-available/default', use_sudo=True)

            sudo('service nginx restart')

            print(green(' * Make Nginx from http to https.'))
            print(green(' * Done'))
            print()

    def common_install_apache2(self):
        """
            Install apache2 web server
        """
        try:
            sudo('apt-get install apache2 -y')
        except Exception as e:
            print(e)

        print(green(' * Installed Apache2 in the system.'))
        print(green(' * Done'))
        print()

    def common_install_python_env(self):
        """
            Install python virtualenv
        """
        sudo('apt-get install python3 python3-pip -y')
        sudo('pip3 install virtualenv')

        run('virtualenv {0}'.format(self.python_env_dir))

        print(green(' * Installed Python3 virtual environment in the system.'))
        print(green(' * Done'))
        print()

    def systemctl_autostart(self, service_name, start_cmd, stop_cmd):
        """
        ubuntu 16.04 systemctl service config
        :param service_name:
        :param start_cmd:
        :param stop_cmd:
        :return:
        """
        # get config content
        service_content = bigdata_conf.systemctl_config.format(
            service_name=service_name,
            start_cmd=start_cmd,
            stop_cmd=stop_cmd
        )

        # write config into file
        with cd('/lib/systemd/system'):
            if not exists(service_name):
                sudo('touch {0}'.format(service_name))
            put(StringIO(service_content), service_name, use_sudo=True)

        # make service auto-start
        sudo('systemctl daemon-reload')
        sudo('systemctl enable {0}'.format(service_name))
        sudo('systemctl start {0}'.format(service_name))

    def java_install(self):
        """
        install java
        :return:
        """
        sudo('apt-get install openjdk-8-jdk -y')

        java_home = run('readlink -f /usr/bin/java | '
                        'sed "s:/jre/bin/java::"')

        append(bigdata_conf.global_env_home, 'export JAVA_HOME={0}'.format(
            java_home
        ), use_sudo=True)
        run('source {0}'.format(bigdata_conf.global_env_home))

    def kafka_install(self):
        """
        kafka download and install
        :return:
        """
        with cd('/tmp'):
            if not exists('kafka.tgz'):
                sudo('wget {0} -O kafka.tgz'.format(
                    bigdata_conf.kafka_download_url
                ))

            sudo('tar -zxf kafka.tgz')

            sudo('rm -rf {0}'.format(bigdata_conf.kafka_home))
            sudo('mv kafka_* {0}'.format(bigdata_conf.kafka_home))

    def kafka_config(self):
        """
        kafka config
        :return:
        """
        uncomment('{0}/config/server.properties'.format(bigdata_conf.kafka_home)
                  , 'listeners=PLAINTEXT', use_sudo=True)
        sed('{0}/config/server.properties'.format(bigdata_conf.kafka_home),
            'PLAINTEXT://.*', 'PLAINTEXT://{0}:9092'.format(env.host_string),
            use_sudo=True)

        self.systemctl_autostart(
            'zookeeper.service',
            '/opt/kafka/bin/zookeeper-server-start.sh /opt/kafka/config/zookeeper.properties',
            '/opt/kafka/bin/zookeeper-server-stop.sh /opt/kafka/config/zookeeper.properties'
        )

        self.systemctl_autostart(
            'kafka.service',
            '/opt/kafka/bin/kafka-server-start.sh /opt/kafka/config/server.properties',
            '/opt/kafka/bin/kafka-server-stop.sh /opt/kafka/config/server.properties'
        )

    def elastic_install(self):
        """
        elasticsearch install
        :return:
        """
        with cd('/tmp'):
            if not exists('elastic.deb'):
                sudo('wget {0} -O elastic.deb'.format(
                    bigdata_conf.elastic_download_url
                ))

            sudo('dpkg -i elastic.deb')
            sudo('apt-get install -y')

    def elastic_config(self):
        """
        config elasticsearch
        :return:
        """
        sudo('systemctl stop elasticsearch.service')
        sudo('systemctl daemon-reload')
        sudo('systemctl enable elasticsearch.service')
        sudo('systemctl start elasticsearch.service')

    def logstash_install(self):
        """
        logstash install
        :return:
        """
        with cd('/tmp'):
            if not exists('logstash.deb'):
                sudo('wget {0} -O logstash.deb'.format(
                    bigdata_conf.logstash_download_url
                ))

            sudo('dpkg -i logstash.deb')
            sudo('apt-get install -y')

    def logstash_config(self):
        """
        config logstash
        :return:
        """
        sudo('systemctl stop logstash.service')
        sudo('systemctl daemon-reload')
        sudo('systemctl enable logstash.service')
        sudo('systemctl start logstash.service')

    def kibana_install(self):
        """
        kibana install
        :return:
        """
        with cd('/tmp'):
            if not exists('kibana.deb'):
                sudo('wget {0} -O kibana.deb'.format(
                    bigdata_conf.kibana_download_url
                ))

            sudo('dpkg -i kibana.deb')
            sudo('apt-get install -y')

    def kibana_config(self):
        """
        config kibana
        :return:
        """

        uncomment("/etc/kibana/kibana.yml", "#server.host:", use_sudo=True)
        sed('/etc/kibana/kibana.yml', 'server.host:.*',
            'server.host: "{0}"'.format(env.host_string), use_sudo=True)
        sudo('systemctl stop kibana.service')
        sudo('systemctl daemon-reload')
        sudo('systemctl enable kibana.service')
        sudo('systemctl start kibana.service')

    def hadoop_install(self):
        """
        install hadoop
        :return:
        """
        with cd('/tmp'):
            if not exists('hadoop.tar.gz'):
                sudo('wget {0} -O hadoop.tar.gz'.format(
                    bigdata_conf.hadoop_download_url
                ))

            sudo('rm -rf hadoop-*')
            sudo('tar -zxf hadoop.tar.gz')
            sudo('rm -rf {0}'.format(bigdata_conf.hadoop_home))
            sudo('mv hadoop-* {0}'.format(bigdata_conf.hadoop_home))

    def spark_install(self):
        """
        download and install spark
        :return:
        """
        sudo('apt-get -y install build-essential python-dev python-six \
             python-virtualenv libcurl4-nss-dev libsasl2-dev libsasl2-modules \
             maven libapr1-dev libsvn-dev zlib1g-dev')

        with cd('/tmp'):
            if not exists('spark.tgz'):
                sudo('wget {0} -O spark.tgz'.format(
                    bigdata_conf.spark_download_url
                ))

            sudo('rm -rf spark-*')
            sudo('tar -zxf spark.tgz')
            sudo('rm -rf {0}'.format(bigdata_conf.spark_home))
            sudo('mv spark-* {0}'.format(bigdata_conf.spark_home))

    def spark_config(self):
        """
        config spark
        :return:
        """
        configs = [
            'export LD_LIBRARY_PATH={0}/lib/native/:$LD_LIBRARY_PATH'.format(
                bigdata_conf.hadoop_home
            ),
            'export SPARK_LOCAL_IP={0}'.format(env.host_string)
        ]

        append(bigdata_conf.global_env_home, configs, use_sudo=True)
        run('source {0}'.format(bigdata_conf.global_env_home))

    def reset_server_env(self, server_name, configure):
        """
        reset server env to server-name
        :param server_name:
        :param configure:
        :return:
        """
        env.host_string = configure[server_name]['host']
        env.user = configure[server_name]['user']
        env.password = configure[server_name]['passwd']

    def generate_ssh(self, server, args, configure):
        """
        异步同时执行SSH生成 generate ssh
        :param server:
        :param args:
        :param configure:
        :return:
        """
        self.reset_server_env(server, configure)

        # chmod project root owner
        sudo('chown {user}:{user} -R {path}'.format(
            user=configure[server]['user'],
            path=bigdata_conf.project_root
        ))

        # generate ssh key
        if not exists('~/.ssh/id_rsa.pub'):
            run('ssh-keygen -t rsa -P "" -f ~/.ssh/id_rsa')

    def add_spark_slave(self, master, slave, configure):
        """
        add spark slave
        :return:
        """
        # go to master server, add config
        self.reset_server_env(master, configure)
        with cd(bigdata_conf.spark_home):
            if not exists('conf/spark-env.sh'):
                sudo('cp conf/spark-env.sh.template conf/spark-env.sh')

            spark_env = bigdata_conf.spark_env.format(
                spark_home=bigdata_conf.spark_home,
                hadoop_home=bigdata_conf.hadoop_home,
                host=env.host_string,
                SPARK_DRIVER_MEMORY=configure[master].get(
                    'SPARK_DRIVER_MEMORY', '512M'
                )
            )

            put(StringIO(spark_env), 'conf/spark-env.sh', use_sudo=True)

            if not exists('conf/slaves'):
                sudo('cp conf/slaves.template conf/slaves')

        # comment slaves localhost
        comment('{0}/conf/slaves'.format(bigdata_conf.spark_home),
                'localhost', use_sudo=True)

        # add slave into config
        append('{0}/conf/slaves'.format(bigdata_conf.spark_home),
               '\n{0}'.format(configure[slave]['host']), use_sudo=True)

        run('scp -r {0} {1}@{2}:/opt'.format(
            bigdata_conf.spark_home,
            configure[slave]['user'],
            configure[slave]['host']
        ))

        # go to slave server
        self.reset_server_env(slave, configure)

        append(bigdata_conf.global_env_home, 'export SPARK_LOCAL_IP={0}'.format(
            configure[slave]['host']
        ), use_sudo=True)
        run('source {0}'.format(bigdata_conf.global_env_home))

        # go to master server, restart server
        self.reset_server_env(master, configure)
        with cd(bigdata_conf.spark_home):
            run('./sbin/stop-master.sh')
            run('./sbin/stop-slaves.sh')
            run('./sbin/start-master.sh')
            run('./sbin/start-slaves.sh')




"""
__init__
"""
from ezhost.BigDataArchi import BigDataArchi
from ezhost.ServerLamp import ServerLamp
from ezhost.ServerLnmp import ServerLnmp
from ezhost.ServerDjango import ServerDjango
from ezhost.ServerLnmpWordpress import ServerLnmpWordpress
from ezhost.ServerDjangoUwsgi import ServerDjangoUwsgi



# -*- coding: utf-8 -*-
"""
BigData Architecture Class

Author: Zhe Xiao
Github: https://github.com/zhexiao/ezhost.git
"""
from ezhost.ServerCommon import ServerCommon

# fabric libs
from fabric.state import output
from fabric.api import run, sudo


# hide exec command
output['running'] = False


class BigDataArchi(ServerCommon):

    def __init__(self, args, **kwargs):
        self.args = args
        self.configure = kwargs.get('configure')

        if self.args.bigdata_app is None:
            raise Exception('请使用`-ba`指定需要安装的应用')

    def install(self):
        if not self.args.skip_master:
            self.update_sys()
            if self.prompt_check("Install Java JDK"):
                self.java_install()

            # check bigdata app
            if self.args.bigdata_app == 'kafka':
                self.install_config_kafka()
            elif self.args.bigdata_app == 'elastic':
                self.install_config_elastic()
            elif self.args.bigdata_app == 'logstash':
                self.install_config_logstash()
            elif self.args.bigdata_app == 'kibana':
                self.install_config_kibana()
            elif self.args.bigdata_app == 'elk':
                self.install_config_elk()
            elif self.args.bigdata_app == 'spark':
                self.install_config_spark()
            else:
                raise Exception('找不到匹配的应用')

        if self.args.add_slave is not None:
            self.add_slave_server()

    def update_sys(self):
        if self.prompt_check("Update system package"):
            self.common_update_sys()

    def install_config_kafka(self):
        """
        install and config kafka
        :return:
        """
        if self.prompt_check("Download and install kafka"):
            self.kafka_install()

        if self.prompt_check("Configure and autostart kafka"):
            self.kafka_config()

    def install_config_elastic(self):
        """
        install and config elasticsearch
        :return:
        """
        if self.prompt_check("Download and install elasticsearch"):
            self.elastic_install()

        if self.prompt_check("Configure and autostart elasticsearch"):
            self.elastic_config()

    def install_config_logstash(self):
        """
        install and config logstash
        :return:
        """
        if self.prompt_check("Download and install logstash"):
            self.logstash_install()

        if self.prompt_check("Configure and autostart logstash"):
            self.logstash_config()

    def install_config_kibana(self):
        """
        install and config kibana
        :return:
        """
        if self.prompt_check("Download and install kibana"):
            self.kibana_install()

        if self.prompt_check("Configure and autostart kibana"):
            self.kibana_config()

    def install_config_elk(self):
        """
        install and config elk
        :return:
        """
        self.install_config_elastic()
        self.install_config_kibana()
        self.install_config_logstash()

    def install_config_spark(self):
        """
        install and config spark
        :return:
        """
        if self.prompt_check("Download and install hadoop"):
            self.hadoop_install()

        if self.prompt_check("Download and install spark"):
            self.spark_install()

        if self.prompt_check("Configure spark"):
            self.spark_config()

    def add_slave_server(self):
        """
        添加slave服务器
        :return:
        """
        master = self.args.config[1]
        slave = self.args.add_slave

        # install java at slave server
        self.reset_server_env(slave, self.configure)
        if self.prompt_check("Update package at slave server"):
            self.common_update_sys()

        if self.prompt_check("Install java and python at slave server"):
            self.java_install()
            sudo('apt-get install -y python python3 python-dev python3-dev')

        # generate ssh key at master server
        if not self.args.skip_master:
            if self.prompt_check("Generate ssh key at Master Server"):
                self.generate_ssh(master, self.args, self.configure)

        # generate ssh key at slave server and make slave connect with master
        if self.prompt_check("Make ssh connection within master and slave"):
            self.generate_ssh(slave, self.args, self.configure)

            # scp slave server ssh key to master server
            run('scp ~/.ssh/id_rsa.pub {0}@{1}:~/.ssh/id_rsa.pub.{2}'.format(
                self.configure[master]['user'],
                self.configure[master]['host'],
                slave
            ))

            # add slave ssh key to master authorized_keys
            self.reset_server_env(master, self.configure)
            run('cat ~/.ssh/id_rsa.pub* >> ~/.ssh/authorized_keys')

            # scp master authorized_keys to slave authorized_keys
            run('scp ~/.ssh/authorized_keys {0}@{1}:~/.ssh'.format(
                self.configure[slave]['user'],
                self.configure[slave]['host']
            ))

        # config slave and master
        if self.prompt_check("Configure master and slave server"):
            master = self.args.config[1]
            slave = self.args.add_slave

            if self.args.bigdata_app == 'spark':
                self.add_spark_slave(master, slave, self.configure)






global_env_home = '/etc/environment'
project_root = '/opt'

kafka_home = '{0}/kafka'.format(project_root)
kafka_download_url = "http://mirrors.tuna.tsinghua.edu.cn/apache/kafka/0.10.2.0/kafka_2.10-0.10.2.0.tgz"

spark_home = '{0}/spark'.format(project_root)
hadoop_home = '{0}/hadoop'.format(project_root)
spark_download_url = "http://mirrors.tuna.tsinghua.edu.cn/apache/spark/spark-2.2.0/spark-2.2.0-bin-hadoop2.7.tgz"
hadoop_download_url = "http://mirrors.tuna.tsinghua.edu.cn/apache/hadoop/common/hadoop-2.7.3/hadoop-2.7.3.tar.gz"

elastic_download_url = "https://artifacts.elastic.co/downloads/elasticsearch/elasticsearch-5.5.0.deb"
logstash_download_url = "https://artifacts.elastic.co/downloads/logstash/logstash-5.5.0.deb"
kibana_download_url = "https://artifacts.elastic.co/downloads/kibana/kibana-5.5.0-amd64.deb"


systemctl_config = """
[Unit]
Description={service_name}
After=syslog.target network.target remote-fs.target nss-lookup.target

[Service]
Type=simple
ExecStart={start_cmd}
ExecStop={stop_cmd}
PrivateTmp=true

[Install]
WantedBy=multi-user.target 
"""

spark_env = """
export SPARK_HOME={spark_home}
export HADOOP_HOME={hadoop_home}
export PATH=$PATH:$JAVA_HOME/bin:$HADOOP_HOME/bin:$HADOOP_HOME/sbin:$SCALA_HOME/bin
export SPARK_LIBARY_PATH=.:$JAVA_HOME/lib:$JAVA_HOME/jre/lib:$HADOOP_HOME/lib/native

export SPARK_MASTER_HOST={host}

SPARK_LOCAL_DIRS={spark_home}
SPARK_DRIVER_MEMORY={SPARK_DRIVER_MEMORY}
"""



# -*- coding: utf-8 -*-
"""
Base class for all server class
"""
from fabric.api import env

from ezhost.ServerAbstract import ServerAbstract
from ezhost.ServerLists import ServerLists
from ezhost.ServerCommand import ServerCommand


class DynamicImporter:
    """
    Dynamic import module and call class
    """
    def __init__(self, module_name, class_name, **kwargs):
        args = kwargs.get('args')
        configure = kwargs.get('configure')

        module_name = __import__(module_name)
        my_class = getattr(module_name, class_name)
        my_class(args, configure=configure).install()


class ServerBase(ServerAbstract):
    """
    Server command bootstrap class
    """

    def __init__(self, args, configure_obj, **kwargs):
        self.args = args
        self.configure_obj = configure_obj
        self.configure = kwargs.get('configure')

        # set common parameters
        self.server_type = self.args.server
        # parser parameters
        self._parse_parameters()

        # initial host
        self.init_host()

        # install
        self.install()

    def _parse_parameters(self):
        # set args from commmand parser
        if self.configure_obj is None:
            self.host_string = self.args.host
            self.host_user = self.args.user
            self.host_passwd = self.args.passwd
            self.host_keyfile = self.args.keyfile

        # set args from config file parser
        else:
            try:
                self.host_string = self.configure_obj['host']
                self.host_user = self.configure_obj['user']
            except:
                raise KeyError('缺少必要的登录信息')

            try:
                self.host_keyfile = self.configure_obj['keyfile']
            except:
                self.host_keyfile = None

            try:
                self.host_passwd = self.configure_obj['passwd']
            except:
                self.host_passwd = None

            if self.host_passwd is None and self.host_keyfile is None:
                raise KeyError('缺少必要的登录信息')

    def init_host(self):
        """
            Initial host
        """
        env.host_string = self.host_string
        env.user = self.host_user
        env.password = self.host_passwd
        env.key_filename = self.host_keyfile

    def install(self):
        """
        install the server
        """
        try:
            if self.args.server is not None:
                server = ServerLists(self.server_type)
                DynamicImporter(
                    'ezhost',
                    server.name,
                    args=self.args,
                    configure=self.configure
                )
            else:
                ServerCommand(self.args)
        except Exception as e:
            raise e



# -*- coding: utf-8 -*-
#!/usr/bin/python
"""
This module contains `main` method plus related subroutines.
`main` is executed as the command line ``ezhost`` program and takes care of
parsing options and commands

The other callables defined in this module are internal only. Anything useful
to individuals leveraging Fabric as a library, should be kept elsewhere.
"""
import argparse
import configparser
from ezhost.ServerBase import ServerBase


def main():
    """
        Check args
    """

    configure_obj = None
    parser = argparse.ArgumentParser(description='自动化安装')
    parser.add_argument(
        '-s', '--server',
        help='服务器代替名',
    )

    parser.add_argument(
        '-ba', '--bigdata-app',
        help='大数据服务应用替代名',
    )

    parser.add_argument(
        '-add-slave', '--add-slave',
        help='添加一个子节点服务器',
    )

    parser.add_argument(
        '-skip-master', '--skip-master',
        dest='skip_master',
        action='store_true',
        help='跳过master服务器的安装过程',
    )

    # force to install packages without ask question
    parser.add_argument(
        '-f', '--force',
        dest='force',
        action='store_true',
        help='不询问是否安装',
    )
    parser.add_argument(
        '-nf', '--not-force',
        dest='force',
        action='store_false',
        help='询问是否安装',
    )
    parser.set_defaults(force=False)

    # login to remote mysql, default is false
    parser.add_argument(
        '-my', '--mysql',
        dest='login_mysql',
        action='store_true',
        help='登录到Mysql数据库',
    )
    parser.set_defaults(login_mysql=False)

    # login to server
    parser.add_argument(
        '-login', '--login',
        dest='login_server',
        action='store_true',
        help='登录到远程服务器',
    )
    parser.set_defaults(login_server=False)

    parser.add_argument(
        '-p', '--project',
        help='项目名称，默认是demo',
        default='demo'
    )

    parser.add_argument(
        '-gp', '--git-pull',
        help='Github项目的保存目录',
    )

    parser.add_argument(
        '-C', '--config',
        help='配置文件路径',
        nargs=2
    )

    parser.add_argument(
        '-H', '--host',
        help='客户机地址',
    )

    parser.add_argument(
        '-U', '--user',
        help='客户机用户名',
    )

    parser.add_argument(
        '-P', '--passwd',
        help='客户机登录密码',
    )

    parser.add_argument(
        '-K', '--keyfile',
        help='客户机SSH KEY的路径',
    )

    parser.add_argument(
        '-ap', '--active-port',
        help='客户机已经开启的端口',
        action='store_true'
    )

    args = parser.parse_args()

    # if config file and host both not provide, throw a error
    if args.config is None and args.host is None:
        raise ValueError('缺少配置文件-C(--config)或客户机地址-H(--host)')

    # check user and passwd
    if args.host is not None:
        if (args.user and (args.passwd or args.keyfile)) is None:
            raise ValueError('缺少登录必要的信息')

    # if exist config file, read configuration from file
    configure = None
    if args.config is not None:
        # init configuration parser
        configure = configparser.ConfigParser()
        configure.read(args.config[0])
        config_sections = configure.sections()

        # check key exist
        if args.config[1] not in config_sections:
            raise KeyError('未找到与{0}对应的登录配置文件，存在的配置文件为{1}'
                           .format(args.config[1], config_sections))

        configure_obj = configure[args.config[1]]
    # init server
    ServerBase(
        args, configure_obj,
        configure=configure
    )



web_dir = '/var/www/html'
web_ssl_dir = '/etc/nginx/ssl'

simple_web_config = """server
{{
    listen 80 default_server;
    listen [::]:80 default_server;

    root {0};
    autoindex on;

    # Add index.php to the list if you are using PHP
    index index.php index.html index.htm index.nginx-debian.html;

    server_name localhost;

    # Don't log robots.txt or favicon.ico files
    location = /favicon.ico {{ log_not_found off; access_log off; }}
    location = /robots.txt  {{ access_log off; log_not_found off; }}

    # pass the PHP scripts to FastCGI server listening on 127.0.0.1:9000
    location ~ \.php$ {{
       include snippets/fastcgi-php.conf;
       # With php5-fpm:
       fastcgi_pass unix:/var/run/php5-fpm.sock;
    }}
}}
"""

simple_php7_web_config = """server
{{
    listen 80 default_server;
    listen [::]:80 default_server;

    root {0};
    autoindex on;

    # Add index.php to the list if you are using PHP
    index index.php index.html index.htm index.nginx-debian.html;

    server_name localhost;

    # Don't log robots.txt or favicon.ico files
    location = /favicon.ico {{ log_not_found off; access_log off; }}
    location = /robots.txt  {{ access_log off; log_not_found off; }}

    # pass the PHP scripts to FastCGI server listening on 127.0.0.1:9000
    location ~ \.php$ {{
       include snippets/fastcgi-php.conf;
       # With php7.0-fpm:
       fastcgi_pass unix:/var/run/php/php7.0-fpm.sock;
    }}
}}
"""

simple_ssl_web_conf = """server
{{
    listen 443 default_server;
    listen [::]:443 default_server;

    root {dt[0]};
    autoindex on;

    # Add index.php to the list if you are using PHP
    index index.php index.html index.htm index.nginx-debian.html;

    server_name localhost;

    # Don't log robots.txt or favicon.ico files
    location = /favicon.ico {{ log_not_found off; access_log off; }}
    location = /robots.txt  {{ access_log off; log_not_found off; }}

    ssl on;
    ssl_certificate {dt[1]}/cert.pem;
    ssl_certificate_key {dt[1]}/cert.key;

    # pass the PHP scripts to FastCGI server listening on 127.0.0.1:9000
    location ~ \.php$ {{
       include snippets/fastcgi-php.conf;
       # With php5-fpm:
       fastcgi_pass unix:/var/run/php5-fpm.sock;
    }}
}}
"""

wordpress_web_conf = """server
{{
    listen 80 default_server;
    listen [::]:80 default_server;

    root {0}/{1};
    autoindex on;

    # Add index.php to the list if you are using PHP
    index index.php index.html index.htm index.nginx-debian.html;

    server_name localhost;
    location / {{
        try_files $uri $uri/ /index.php?q=$uri&$args;
    }}

    # Don't log robots.txt or favicon.ico files
    location = /favicon.ico {{ log_not_found off; access_log off; }}
    location = /robots.txt  {{ access_log off; log_not_found off; }}

    # pass the PHP scripts to FastCGI server listening on 127.0.0.1:9000
    location ~ \.php$ {{
       include snippets/fastcgi-php.conf;
       # With php5-fpm:
       fastcgi_pass unix:/var/run/php5-fpm.sock;
       fastcgi_split_path_info ^(.+\.php)(/.+)$;
    }}
}}
"""

wordpress_php7_web_conf = """server
{{
    listen 80 default_server;
    listen [::]:80 default_server;

    root {0}/{1};
    autoindex on;

    # Add index.php to the list if you are using PHP
    index index.php index.html index.htm index.nginx-debian.html;

    server_name localhost;
    location / {{
        try_files $uri $uri/ /index.php?q=$uri&$args;
    }}

    # Don't log robots.txt or favicon.ico files
    location = /favicon.ico {{ log_not_found off; access_log off; }}
    location = /robots.txt  {{ access_log off; log_not_found off; }}

    # pass the PHP scripts to FastCGI server listening on 127.0.0.1:9000
    location ~ \.php$ {{
       include snippets/fastcgi-php.conf;
       # With php7-fpm:
       fastcgi_pass unix:/var/run/php/php7.0-fpm.sock;
       fastcgi_split_path_info ^(.+\.php)(/.+)$;
    }}
}}
"""



from enum import Enum


class ServerLists(Enum):
    """
        All valid server lists and shortcut
    """

    ServerLamp = 'lamp'
    ServerLnmp = 'lnmp'
    ServerDjango = 'django'
    ServerDjangoUwsgi = 'django-uwsgi'
    ServerLnmpWordpress = 'lnmp-wordpress'
    BigDataArchi = 'bigdata'



# -*- coding: utf-8 -*-
"""
This class is aim to install Django Applications with uWSGI and Nginx,
we will using linux supervisor to control the server auto start.

Usage:
    $ sudo pip install ezhost
    $ ezhost -s django-uwsgi -H 127.0.0.1:2200 -U vagrant -P vagrant

Author: Zhe Xiao
Github: https://github.com/zhexiao/ezhost.git
"""

from io import StringIO

# fabric libs
from fabric.colors import red, green
from fabric.api import prompt, run, sudo, put
from fabric.contrib.files import exists
from fabric.state import output
from fabric.context_managers import cd

from ezhost.ServerCommon import ServerCommon
from ezhost.ServerDjango import ServerDjango

# hide exec command
output['running'] = False


class ServerDjangoUwsgi(ServerCommon):
    def __init__(self, args, **kwargs):
        self.args = args
        self.project = self.args.project
        self.project_dir = '{0}/{1}'.format(self.nginx_web_dir, self.project)

    def install(self):
        # install basic django server
        ServerDjango(self.args).install()

        self.install_nginx()
        self.install_uwsgi()
        self.install_supervisor()

    def install_nginx(self):
        if self.args.force or prompt(red(' * Install Nginx (y/n)?'), default='y') == 'y':
            self.common_install_nginx()

            # nginx configuration
            put(StringIO(self.django_uwsgi_with_nginx), '/etc/nginx/sites-enabled/default', use_sudo=True)

            # restart server
            sudo('service nginx restart')

    def install_uwsgi(self):
        if self.args.force or prompt(red(' * Install Uwsgi service (y/n)?'), default='y') == 'y':
            sudo('pip3 install uwsgi')

            # uwsgi config need real env path
            with cd(self.python_env_dir):
                real_env_path = run('pwd')

            # get user
            home_user = run('echo $USER')

            # uwsgi config string
            django_uwsgi_ini = self.django_uwsgi_ini.format(self.nginx_web_dir, self.project, real_env_path, home_user)

            # modify uwsgi config file
            with cd(self.project_dir):
                if not exists('{0}.ini'.format(self.project)):
                    run('touch {0}.ini'.format(self.project))

                put(StringIO(django_uwsgi_ini), '{0}.ini'.format(self.project), use_sudo=True)

            print(green(' * Installed Uwsgi service in the system.'))
            print(green(' * Done '))
            print()

    def install_supervisor(self):
        if self.args.force or prompt(red(' * Install Supervisor controller for Uwsgi (y/n)?'), default='y') == 'y':
            sudo('apt-get install supervisor -y')

            # supervisor config string
            supervisor_uwsgi_ini = self.supervisor_uwsgi_ini.format(self.project, self.project_dir)

            # create supervisor control configruation
            with cd(self.supervisor_config_dir):
                if not exists('{0}_sysd.conf'.format(self.project)):
                    sudo('touch {0}_sysd.conf'.format(self.project))

                # supervisor control uwsgi config
                put(StringIO(supervisor_uwsgi_ini), '{0}_sysd.conf'.format(self.project), use_sudo=True)

            # create supervisor log and error file
            with cd('/var/log'):
                if not exists('{0}_out.log'.format(self.project)):
                    sudo('touch {0}_out.log'.format(self.project))

                if not exists('{0}_error.log'.format(self.project)):
                    sudo('touch {0}_error.log'.format(self.project))

            # enable and start supervisor
            try:
                # ubuntu 16
                sudo('systemctl enable supervisor')
                sudo('systemctl start supervisor')
            except:
                # ubuntu 14
                sudo('supervisorctl reread')
                sudo('supervisorctl update')
                sudo('update-rc.d supervisor enable')

            print(green(' * Installed Supervisor controller in the system.'))
            print(green(' * Done '))
            print()



# -*- coding: utf-8 -*-
import os
from abc import ABCMeta, abstractmethod, abstractproperty
import ezhost.config.apache_conf_string as apache_conf_string
import ezhost.config.php_conf_string as php_conf_string
import ezhost.config.nginx_conf_string as nginx_conf_string
import ezhost.config.django_conf_string as django_conf_string
import ezhost.config.bigdata_conf_string as bigdata_conf

class ServerAbstract(metaclass=ABCMeta):
    """
        Server abstract class, all server should extend from it
    """

    def __init__(self):
        # host information
        self._host_string = None
        self._host_user = None
        self._host_passwd = None
        self._host_keyfile = None
        # server type
        self._server_type = None
        self._project = None

    @property
    def host_string(self):
        """
            Returns:
                str: hostname string
        """
        return self._host_string

    @host_string.setter
    def host_string(self, value):
        """
            Host string setter
        """
        self._host_string = value

    @property
    def host_user(self):
        """
            Host user getter
        """
        return self._host_user

    @host_user.setter
    def host_user(self, value):
        """
            Host user setter
        """
        self._host_user = value

    @property
    def host_passwd(self):
        """
            Host password getter
        """
        return self._host_passwd

    @host_passwd.setter
    def host_passwd(self, value):
        """
            Host password setter
        """
        self._host_passwd = value

    @property
    def host_keyfile(self):
        """
            Host keyfile getter
        """
        return self._host_keyfile

    @host_keyfile.setter
    def host_keyfile(self, value):
        """
            Host keyfile setter
        """
        self._host_keyfile = value

    @property
    def server_type(self):
        """
            Server type getter
        """
        return self._server_type

    @server_type.setter
    def server_type(self, value):
        """
            Server type setter
        """
        self._server_type = value

    @property
    def project(self):
        """
            project getter
        """
        return self._project

    @project.setter
    def project(self, value):
        """
            project setter
        """
        self._project = value

    @property
    def command_path(self):
        """
            Currently command path
        """
        return os.path.dirname(os.path.realpath(__file__))

    @property
    def mysql_password(self):
        """
            Mysql password
        """
        return 'password'

    @property
    def apache_web_dir(self):
        """
            default apache project dir
        """

        return apache_conf_string.web_dir

    @property
    def nginx_web_dir(self):
        """
            default apache project dir
        """

        return nginx_conf_string.web_dir

    @property
    def apache_dir_index(self):
        """
            Currently, if a user requests a directory from the server, Apache
            will first look for a file called index.html. We want to tell our
            web server to prefer PHP files, so we'll make Apache look for an
            index.php file first.
        """

        return apache_conf_string.module_index_str

    @property
    def phpinfo(self):
        """
           In order to test that our system is configured properly for PHP,
           we can create a very basic PHP script.
        """

        return php_conf_string.phpinfo_str

    @property
    def nginx_web_config(self):
        """
           Nginx web config
        """

        return nginx_conf_string.simple_web_config.format(self.nginx_web_dir)

    @property
    def nginx_php7_web_config(self):
        """
           Nginx web config
        """

        return nginx_conf_string.simple_php7_web_config.format(self.nginx_web_dir)

    @property
    def nginx_web_ssl_config(self):
        """
           Nginx web ssl config
        """

        dt = [self.nginx_web_dir, self.nginx_ssl_dir]
        return nginx_conf_string.simple_ssl_web_conf.format(dt=dt)

    @property
    def python_env_dir(self):
        """
            Python virtualenv dir
        """
        return '~/.{0}'.format(self.project)

    @property
    def django_uwsgi_ini(self):
        """
           This ini file is django uwsgi configuration
        """

        return django_conf_string.uwsgi_ini_conf

    @property
    def django_uwsgi_with_nginx(self):
        """
           This ini file nginx configuration with uwsgi application
        """

        return django_conf_string.uwsgi_nginx_conf.format(self.project)

    @property
    def supervisor_uwsgi_ini(self):
        """
           supervisor control uwsgi configuration
        """

        return django_conf_string.uwsgi_supervisor_conf

    @property
    def supervisor_etc_dir(self):
        """
            supervisor control config dir
        """
        return django_conf_string.supervisor_dir

    @property
    def supervisor_config_dir(self):
        """
            supervisor control config dir
        """
        return django_conf_string.supervisor_conf_dir

    @property
    def nginx_ssl_dir(self):
        """
            ssl directory
        """
        return nginx_conf_string.web_ssl_dir

    @property
    def nginx_web_wordpress_config(self):
        """
           Nginx web wordpress config
        """

        return nginx_conf_string.wordpress_web_conf.format(
            self.nginx_web_dir, self.project
        )

    @property
    def nginx_php7_web_wordpress_config(self):
        """
           Nginx web wordpress config
        """

        return nginx_conf_string.wordpress_php7_web_conf.format(
            self.nginx_web_dir, self.project
        )

    @property
    def kafka_install_dir(self):
        """
        kafka install dir
        :return:
        """
        return bigdata_conf.kafka_home

    @abstractmethod
    def install(self):
        """
            All subclass need implement this function
        """
        pass



# -*- coding: utf-8 -*-
"""
Install basic django project

Usage:
    $ -s django or -server django

Author: Zhe Xiao
Github: https://github.com/zhexiao/ezhost.git
"""

from ezhost.ServerCommon import ServerCommon

# fabric libs
from fabric.colors import red, green
from fabric.api import prompt, run, sudo
from fabric.state import output
from fabric.context_managers import cd
from fabric.contrib.files import exists

# hide exec command
output['running'] = False


class ServerDjango(ServerCommon):

    def __init__(self, args, **kwargs):
        self.args = args
        self.project = self.args.project

    def install(self):
        self.update_sys()
        self.install_mysql()
        self.install_virtualenv()
        self.install_packages()
        self.install_project()

    def update_sys(self):
        if self.args.force or prompt(red(' * Update system package (y/n)?'), default='y') == 'y':
            self.common_update_sys()

    def install_mysql(self):
        if self.args.force or prompt(red(' * Install MySql (y/n)?'), default='y') == 'y':
            # python pip mysqlclient dependency
            sudo('apt-get install python3-dev libmysqlclient-dev -y')

            self.common_install_mysql()

    def install_virtualenv(self):
        if self.args.force or prompt(red(' * Install Python3 virtual environment (y/n)?'), default='y') == 'y':
            self.common_install_python_env()

    def install_packages(self):
        if self.args.force or prompt(red(' * Install Python3 packages into virtual environment (y/n)?'), default='y') == 'y':
            # go to virtualenv folder
            with cd(self.python_env_dir):
                run('./bin/pip install django djangorestframework django-filter markdown mysqlclient')
                run('./bin/pip freeze')

                print(green(' * Installed Python3 packages into virtual environment.'))
                print(green(' * Done '))
                print()

    def install_project(self):
        if self.args.force or prompt(red(' * Install Django project at /var/www/html (y/n)?'), default='y') == 'y':
            # check project folder
            if not exists(self.nginx_web_dir):
                sudo('mkdir -p {0}'.format(self.nginx_web_dir))

            # go to project folder and create django project
            with cd(self.nginx_web_dir):
                sudo('{0}/bin/django-admin startproject {1}'.format(self.python_env_dir, self.project))
                sudo('chmod -R 777 {0}/{1}'.format(self.nginx_web_dir, self.project))

            print(green(' * Installed Django project at /var/www/html.'))
            print(green(' * Done '))
            print()



# -*- coding: utf-8 -*-
"""
This class is aim to install lnmp server(Linux, Nginx, Mysql and PHP) into your
local environment.

Usage:
    $ sudo pip install ezhost
    $ ezhost -s lnmp -H 127.0.0.1:2201 -U vagrant -P vagrant

Author: Zhe Xiao
Github: https://github.com/zhexiao/ezhost.git
"""

from io import StringIO

from ezhost.ServerCommon import ServerCommon

# fabric libs
from fabric.colors import red, green
from fabric.api import prompt, sudo, put
from fabric.contrib.files import sed
from fabric.state import output

# hide exec command
output['running'] = False


class ServerLnmp(ServerCommon):

    def __init__(self, args, **kwargs):
        self.args = args

    def install(self):
        self.update_sys()
        self.install_mysql()
        self.install_nginx()
        self.install_php()

    def update_sys(self):
        if self.args.force or prompt(red(' * Update system package (y/n)?'), default='y') == 'y':
            self.common_update_sys()

    def install_mysql(self):
        if self.args.force or prompt(red(' * Install MySql (y/n)?'), default='y') == 'y':
            self.common_install_mysql()

    def install_nginx(self):
        if self.args.force or prompt(red(' * Install Nginx (y/n)?'), default='y') == 'y':
            self.common_install_nginx()

    def install_php(self):
        if self.args.force or prompt(red(' * Install PHP (y/n)?'), default='y') == 'y':
            # try install php7 or php5
            try:
                sudo('apt-get install php7.0-fpm php7.0-mysql php7.0-gd php7.0-curl -y')

                # Find the line, cgi.fix_pathinfo=1, and change the 1 to 0.
                sed('/etc/php/7.0/fpm/php.ini', ';cgi.fix_pathinfo=1', 'cgi.fix_pathinfo=0', use_sudo=True)
                # do nginx config
                put(StringIO(self.nginx_php7_web_config), '/etc/nginx/sites-available/default', use_sudo=True)

                sudo('service nginx restart')
                sudo('service php7.0-fpm restart')
                print(green(' * Installed php7.0 and php7-mysql in the system.'))
            except:
                sudo('apt-get install php5-fpm php5-mysql php5-gd php5-curl -y')

                # Find the line, cgi.fix_pathinfo=1, and change the 1 to 0.
                sed('/etc/php5/fpm/php.ini', ';cgi.fix_pathinfo=1', 'cgi.fix_pathinfo=0', use_sudo=True)
                # do nginx config
                put(StringIO(self.nginx_web_config), '/etc/nginx/sites-available/default', use_sudo=True)

                sudo('service nginx restart')
                sudo('service php5-fpm restart')
                print(green(' * Installed php5 and php5-mysql in the system.'))

            # write phpinfo for test
            put(StringIO(self.phpinfo), '{0}/info.php'.format(self.nginx_web_dir), use_sudo=True)

            print(green(' * Done'))
            print()
